@@ -25,7 +25,6 @@ import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.HashMap;
-import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Stack;
@@ -45,6 +44,7 @@ import com.eteks.sweethome3d.model.CatalogTexture;
 import com.eteks.sweethome3d.model.Compass;
 import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.DimensionLine;
+import com.eteks.sweethome3d.model.FurnitureCategory;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeDoorOrWindow;
 import com.eteks.sweethome3d.model.HomeEnvironment;
@@ -63,6 +63,7 @@ import com.eteks.sweethome3d.model.Polyline;
 import com.eteks.sweethome3d.model.Room;
 import com.eteks.sweethome3d.model.Sash;
 import com.eteks.sweethome3d.model.TextStyle;
+import com.eteks.sweethome3d.model.TexturesCategory;
 import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.model.Wall;
 import com.eteks.sweethome3d.tools.ResourceURLContent;
@@ -221,7 +222,9 @@ import com.eteks.sweethome3d.tools.ResourceURLContent;
  *       icon CDATA #IMPLIED
  *       planIcon CDATA #IMPLIED
  *       modelRotation CDATA "1 0 0 0 1 0 0 0 1"
+ *       modelCenteredAtOrigin CDATA #IMPLIED
  *       backFaceShown (false | true) "false"
+ *       modelSize CDATA #IMPLIED
  *       doorOrWindow (false | true) "false"
  *       resizable (false | true) "true"
  *       deformable (false | true) "true"
@@ -232,11 +235,20 @@ import com.eteks.sweethome3d.tools.ResourceURLContent;
  *       creator CDATA #IMPLIED
  *       valueAddedTaxPercentage CDATA #IMPLIED
  *       currency CDATA #IMPLIED'>
+ *
+ * &lt;!ENTITY % pieceOfFurnitureHorizontalRotationAttributes
+ *      'horizontallyRotatable (false | true) "true"
+ *       pitch CDATA "0"
+ *       roll CDATA "0"
+ *       widthInPlan CDATA #IMPLIED
+ *       depthInPlan CDATA #IMPLIED
+ *       heightInPlan CDATA #IMPLIED'>
  * 
  * &lt;!ELEMENT pieceOfFurniture (property*, textStyle?, texture?, material*)>
  * &lt;!ATTLIST pieceOfFurniture 
  *       %furnitureCommonAttributes;
- *       %pieceOfFurnitureCommonAttributes;>
+ *       %pieceOfFurnitureCommonAttributes;
+ *       %pieceOfFurnitureHorizontalRotationAttributes;>
  * 
  * &lt;!ELEMENT doorOrWindow (sash*, property*, textStyle?, texture?, material*)>
  * &lt;!ATTLIST doorOrWindow 
@@ -244,6 +256,8 @@ import com.eteks.sweethome3d.tools.ResourceURLContent;
  *       %pieceOfFurnitureCommonAttributes;
  *       wallThickness CDATA "1"
  *       wallDistance CDATA "0"
+ *       wallCutOutOnBothSides (false | true) "false"
+ *       widthDepthDeformable (false | true) "true"
  *       cutOutShape CDATA #IMPLIED
  *       boundToWall (false | true) "true">
  * 
@@ -259,6 +273,7 @@ import com.eteks.sweethome3d.tools.ResourceURLContent;
  * &lt;!ATTLIST light 
  *       %furnitureCommonAttributes;
  *       %pieceOfFurnitureCommonAttributes;
+ *       %pieceOfFurnitureHorizontalRotationAttributes;
  *       power CDATA "0.5">
  * 
  * &lt;!ELEMENT lightSource EMPTY>
@@ -285,6 +300,8 @@ import com.eteks.sweethome3d.tools.ResourceURLContent;
  *       width CDATA #REQUIRED
  *       height CDATA #REQUIRED
  *       angle CDATA "0"
+ *       scale CDATA "1"
+ *       creator CDATA #IMPLIED
  *       leftToRightOriented (true | false) "true"
  *       image CDATA #REQUIRED>
  *       
@@ -403,8 +420,8 @@ public class HomeXMLHandler extends DefaultHandler {
   private Baseboard rightSideBaseboard;
   private BackgroundImage homeBackgroundImage;
   private BackgroundImage backgroundImage;
-  private final Map<String, String> homeProperties = new LinkedHashMap<String, String>();
-  private final Map<String, String> properties = new LinkedHashMap<String, String>();
+  private final Map<String, String> homeProperties = new HashMap<String, String>();
+  private final Map<String, String> properties = new HashMap<String, String>();
   private final Map<String, TextStyle>    textStyles = new HashMap<String, TextStyle>();
   private final Map<String, HomeTexture>  textures = new HashMap<String, HomeTexture>();
   private final List<HomeMaterial> materials = new ArrayList<HomeMaterial>();
@@ -414,6 +431,8 @@ public class HomeXMLHandler extends DefaultHandler {
   private final List<float[]>      points = new ArrayList<float[]>();
   private final List<HomePieceOfFurniture.SortableProperty> furnitureVisibleProperties = new ArrayList<HomePieceOfFurniture.SortableProperty>();
   
+  private static final String UNIQUE_ATTRIBUTE = "@&unique&@";
+  
   public HomeXMLHandler() {
     this(null);
   }
@@ -498,7 +517,7 @@ public class HomeXMLHandler extends DefaultHandler {
       this.leftSideBaseboard = null;
       this.rightSideBaseboard = null;
     } else if ("baseboard".equals(name)) {
-      this.textures.remove(null);
+      this.textures.remove(UNIQUE_ATTRIBUTE);
     } else if ("material".equals(name)) {
       this.materialTexture = null;
     }
@@ -634,12 +653,16 @@ public class HomeXMLHandler extends DefaultHandler {
     } else if ("text".equals(name)) {
       this.labelText = getCharacters();
     } else if ("textStyle".equals(name)) {
-      this.textStyles.put(attributesMap.get("attribute"), createTextStyle(attributesMap));
+      String attribute = attributesMap.get("attribute");
+      this.textStyles.put(attribute != null ? attribute : UNIQUE_ATTRIBUTE, 
+          createTextStyle(attributesMap));
     } else if ("texture".equals(name)) {
       if ("material".equals(parent)) {
         this.materialTexture = createTexture(attributesMap); 
       } else {
-        this.textures.put(attributesMap.get("attribute"), createTexture(attributesMap));
+        String attribute = attributesMap.get("attribute");
+        this.textures.put(attribute != null ? attribute : UNIQUE_ATTRIBUTE, 
+            createTexture(attributesMap));
       }
     } else if ("material".equals(name)) {
       this.materials.add(createMaterial(attributesMap));
@@ -663,7 +686,7 @@ public class HomeXMLHandler extends DefaultHandler {
           parseOptionalFloat(attributesMap, "diameter")));
     } else if ("backgroundImage".equals(name)) {
       BackgroundImage backgroundImage = new BackgroundImage(
-          parseContent(attributesMap.get("image")), 
+          parseContent(attributesMap.get("image"), null), 
           parseFloat(attributesMap, "scaleDistance"), 
           parseFloat(attributesMap, "scaleDistanceXStart"), 
           parseFloat(attributesMap, "scaleDistanceYStart"), 
@@ -721,13 +744,24 @@ public class HomeXMLHandler extends DefaultHandler {
   
   /**
    * Returns a new {@link Home} instance initialized from the given <code>attributes</code>.
+   * @return a home instance with its version set. 
    */
   private Home createHome(Map<String, String> attributes) throws SAXException {
+    Home home;
     if (attributes.get("wallHeight") != null) {
-      return new Home(parseFloat(attributes, "wallHeight"));
+      home = new Home(parseFloat(attributes, "wallHeight"));
     } else {
-      return new Home();
+      home = new Home();
     }
+    String version = attributes.get("version");
+    if (version != null) {
+      try {
+        home.setVersion(Integer.parseInt(version));
+      } catch (NumberFormatException ex) {
+        throw new SAXException("Invalid value for integer attribute version", ex);
+      }
+    }
+    return home;
   }
   
   /**
@@ -744,14 +778,6 @@ public class HomeXMLHandler extends DefaultHandler {
       this.home.setFurnitureVisibleProperties(this.furnitureVisibleProperties);
     }
     this.home.setBackgroundImage(this.homeBackgroundImage);
-    String version = attributes.get("version");
-    if (version != null) {
-      try {
-        home.setVersion(Integer.parseInt(version));
-      } catch (NumberFormatException ex) {
-        throw new SAXException("Invalid value for integer attribute version", ex);
-      }
-    }
     home.setName(attributes.get("name"));
     String selectedLevelId = attributes.get("selectedLevel");
     if (selectedLevelId != null) {
@@ -858,7 +884,7 @@ public class HomeXMLHandler extends DefaultHandler {
   }
 
   /**
-   * Returns a new {@link Print} instance initialized from the given <code>attributes</code>.
+   * Returns a new {@link HomePrint} instance initialized from the given <code>attributes</code>.
    */
   protected HomePrint createPrint(Map<String, String> attributes) throws SAXException {
     HomePrint.PaperOrientation paperOrientation = HomePrint.PaperOrientation.PORTRAIT;
@@ -998,6 +1024,7 @@ public class HomeXMLHandler extends DefaultHandler {
    * Returns a new {@link HomePieceOfFurniture} instance initialized from the given <code>attributes</code>.
    */
   private HomePieceOfFurniture createPieceOfFurniture(String elementName, Map<String, String> attributes) throws SAXException {
+    String catalogId = attributes.get("catalogId");
     String [] tags = attributes.get("tags") != null 
         ? attributes.get("tags").split(" ")
         : null;
@@ -1044,16 +1071,16 @@ public class HomeXMLHandler extends DefaultHandler {
         cutOutShape = "M0,0 v1 h1 v-1 z";
       }
       return new HomeDoorOrWindow(new CatalogDoorOrWindow(
-          attributes.get("catalogId"), 
+          catalogId, 
           attributes.get("name"), 
           attributes.get("description"), 
           attributes.get("information"), 
           tags, 
           parseOptionalLong(attributes, "creationDate"), 
           parseOptionalFloat(attributes, "grade"), 
-          parseContent(attributes.get("icon")),  
-          parseContent(attributes.get("planIcon")), 
-          parseContent(attributes.get("model")), 
+          parseContent(attributes.get("icon"), catalogId),  
+          parseContent(attributes.get("planIcon"), catalogId), 
+          parseContent(attributes.get("model"), catalogId), 
           parseFloat(attributes, "width"), 
           parseFloat(attributes, "depth"), 
           parseFloat(attributes, "height"), 
@@ -1063,9 +1090,12 @@ public class HomeXMLHandler extends DefaultHandler {
           cutOutShape, 
           wallThickness,
           wallDistance,
+          "true".equals(attributes.get("wallCutOutOnBothSides")),
+          !"false".equals(attributes.get("widthDepthDeformable")),
           this.sashes.toArray(new Sash [this.sashes.size()]),
           modelRotation, 
           "true".equals(attributes.get("backFaceShown")),
+          parseOptionalLong(attributes, "modelSize"),
           attributes.get("creator"), 
           !"false".equals(attributes.get("resizable")), 
           !"false".equals(attributes.get("deformable")), 
@@ -1075,16 +1105,16 @@ public class HomeXMLHandler extends DefaultHandler {
           attributes.get("currency")));
     } else if ("light".equals(elementName)) {
       return new HomeLight(new CatalogLight(
-          attributes.get("catalogId"), 
+          catalogId, 
           attributes.get("name"), 
           attributes.get("description"), 
           attributes.get("information"), 
           tags, 
           parseOptionalLong(attributes, "creationDate"), 
           parseOptionalFloat(attributes, "grade"), 
-          parseContent(attributes.get("icon")),  
-          parseContent(attributes.get("planIcon")), 
-          parseContent(attributes.get("model")), 
+          parseContent(attributes.get("icon"), catalogId),  
+          parseContent(attributes.get("planIcon"), catalogId), 
+          parseContent(attributes.get("model"), catalogId), 
           parseFloat(attributes, "width"), 
           parseFloat(attributes, "depth"), 
           parseFloat(attributes, "height"), 
@@ -1095,25 +1125,27 @@ public class HomeXMLHandler extends DefaultHandler {
           attributes.get("staircaseCutOutShape"), 
           modelRotation, 
           "true".equals(attributes.get("backFaceShown")),
+          parseOptionalLong(attributes, "modelSize"),
           attributes.get("creator"), 
           !"false".equals(attributes.get("resizable")), 
           !"false".equals(attributes.get("deformable")), 
           !"false".equals(attributes.get("texturable")), 
+          !"false".equals(attributes.get("horizontallyRotatable")),
           parseOptionalDecimal(attributes, "price"), 
           parseOptionalDecimal(attributes, "valueAddedTaxPercentage"), 
           attributes.get("currency")));
     } else {
       return new HomePieceOfFurniture(new CatalogPieceOfFurniture(
-          attributes.get("catalogId"), 
+          catalogId, 
           attributes.get("name"), 
           attributes.get("description"), 
           attributes.get("information"), 
           tags, 
           parseOptionalLong(attributes, "creationDate"), 
           parseOptionalFloat(attributes, "grade"), 
-          parseContent(attributes.get("icon")),  
-          parseContent(attributes.get("planIcon")), 
-          parseContent(attributes.get("model")), 
+          parseContent(attributes.get("icon"), catalogId),  
+          parseContent(attributes.get("planIcon"), catalogId), 
+          parseContent(attributes.get("model"), catalogId), 
           parseFloat(attributes, "width"), 
           parseFloat(attributes, "depth"), 
           parseFloat(attributes, "height"), 
@@ -1123,10 +1155,12 @@ public class HomeXMLHandler extends DefaultHandler {
           attributes.get("staircaseCutOutShape"), 
           modelRotation, 
           "true".equals(attributes.get("backFaceShown")),
+          parseOptionalLong(attributes, "modelSize"),
           attributes.get("creator"), 
           !"false".equals(attributes.get("resizable")), 
           !"false".equals(attributes.get("deformable")), 
           !"false".equals(attributes.get("texturable")), 
+          !"false".equals(attributes.get("horizontallyRotatable")),
           parseOptionalDecimal(attributes, "price"), 
           parseOptionalDecimal(attributes, "valueAddedTaxPercentage"), 
           attributes.get("currency")));
@@ -1183,6 +1217,32 @@ public class HomeXMLHandler extends DefaultHandler {
       if (angle != null) {
         piece.setAngle(angle);
       }
+      if (piece.isHorizontallyRotatable()) {
+        Float pitch = parseOptionalFloat(attributes, "pitch");
+        if (pitch != null) {
+          piece.setPitch(pitch);
+        }
+        Float roll = parseOptionalFloat(attributes, "roll");
+        if (roll != null) {
+          piece.setRoll(roll);
+        }
+      }
+      Float widthInPlan = parseOptionalFloat(attributes, "widthInPlan");
+      if (widthInPlan != null) {
+        piece.setWidthInPlan(widthInPlan);
+      }
+      Float depthInPlan = parseOptionalFloat(attributes, "depthInPlan");
+      if (depthInPlan != null) {
+        piece.setDepthInPlan(depthInPlan);
+      }
+      Float heightInPlan = parseOptionalFloat(attributes, "heightInPlan");
+      if (heightInPlan != null) {
+        piece.setHeightInPlan(heightInPlan);
+      }
+      if (this.home.getVersion() < 5500 || "false".equals(attributes.get("modelCenteredAtOrigin"))) {
+        // Set value to false only if model rotation matrix is defined
+        piece.setModelCenteredAtOrigin(attributes.get("modelRotation") == null);
+      }
       if (piece.isResizable()) {
         // Attribute already set for HomeFurnitureGroup instances during creation
         piece.setModelMirrored("true".equals(attributes.get("modelMirrored")));
@@ -1196,7 +1256,7 @@ public class HomeXMLHandler extends DefaultHandler {
         if (color != null) {
           piece.setColor(color);
         }
-        HomeTexture texture = this.textures.get(null);
+        HomeTexture texture = this.textures.get(UNIQUE_ATTRIBUTE);
         if (texture != null) {
           piece.setTexture(texture);
         }
@@ -1219,13 +1279,13 @@ public class HomeXMLHandler extends DefaultHandler {
   /**
    * Returns a new {@link Wall} instance initialized from the given <code>attributes</code>.
    */
-  @SuppressWarnings("deprecation")
   private Wall createWall(Map<String, String> attributes) throws SAXException {
     return new Wall(parseFloat(attributes, "xStart"),
         parseFloat(attributes, "yStart"),
         parseFloat(attributes, "xEnd"),
         parseFloat(attributes, "yEnd"),
-        parseFloat(attributes, "thickness"));
+        parseFloat(attributes, "thickness"),
+        0);
   }
 
   /**
@@ -1238,10 +1298,7 @@ public class HomeXMLHandler extends DefaultHandler {
     setProperties(wall);
     wall.setLeftSideBaseboard(this.leftSideBaseboard);
     wall.setRightSideBaseboard(this.rightSideBaseboard);
-    Float height = parseOptionalFloat(attributes, "height");
-    if (height != null) {
-      wall.setHeight(height);
-    }
+    wall.setHeight(parseOptionalFloat(attributes, "height"));
     wall.setHeightAtEnd(parseOptionalFloat(attributes, "heightAtEnd"));
     wall.setArcExtent(parseOptionalFloat(attributes, "arcExtent"));
     wall.setTopColor(parseOptionalColor(attributes, "topColor"));
@@ -1431,7 +1488,7 @@ public class HomeXMLHandler extends DefaultHandler {
                                     String elementName, 
                                     Map<String, String> attributes) throws SAXException {
     setProperties(label);
-    label.setStyle(this.textStyles.get(null));
+    label.setStyle(this.textStyles.get(UNIQUE_ATTRIBUTE));
     Float angle = parseOptionalFloat(attributes, "angle");
     if (angle != null) {
       label.setAngle(angle);
@@ -1455,7 +1512,7 @@ public class HomeXMLHandler extends DefaultHandler {
     return Baseboard.getInstance(parseFloat(attributes, "thickness"), 
         parseFloat(attributes, "height"), 
         parseOptionalColor(attributes, "color"),
-        this.textures.get(null));
+        this.textures.get(UNIQUE_ATTRIBUTE));
   }
   
   /**
@@ -1472,16 +1529,19 @@ public class HomeXMLHandler extends DefaultHandler {
    * Returns a new {@link HomeTexture} instance initialized from the given <code>attributes</code>.
    */
   private HomeTexture createTexture(Map<String, String> attributes) throws SAXException {
-    return new HomeTexture(new CatalogTexture(
-            attributes.get("catalogId"), 
-            attributes.get("name"), 
-            parseContent(attributes.get("image")), 
-            parseFloat(attributes, "width"), 
-            parseFloat(attributes, "height"), 
-            null),
+    String catalogId = attributes.get("catalogId");
+    return new HomeTexture(new CatalogTexture(catalogId, 
+                               attributes.get("name"), 
+                               parseContent(attributes.get("image"), catalogId), 
+                               parseFloat(attributes, "width"), 
+                               parseFloat(attributes, "height"), 
+                               attributes.get("creator")),
         attributes.get("angle") != null
             ? parseFloat(attributes, "angle")
             : 0,
+        attributes.get("scale") != null
+            ? parseFloat(attributes, "scale")
+            : 1,
         !"false".equals(attributes.get("leftToRightOriented")));
   }
 
@@ -1514,7 +1574,7 @@ public class HomeXMLHandler extends DefaultHandler {
     if (color != null) {
       try {
         // Need to use parseLong in case parsed number is bigger than 2^31
-        return new Integer((int)Long.parseLong(color, 16));
+        return Integer.valueOf((int)Long.parseLong(color, 16));
       } catch (NumberFormatException ex) {
         throw new SAXException("Invalid value for color attribute " + name, ex);
       }
@@ -1591,25 +1651,60 @@ public class HomeXMLHandler extends DefaultHandler {
   /**
    * Returns the content object matching the given string.
    */
-  private Content parseContent(String content) throws SAXException {
-    if (content != null) {
+  private Content parseContent(String contentFile, String catalogId) throws SAXException {
+    if (contentFile != null) {
       try {
-        return new ResourceURLContent(new URL(content), content.startsWith("jar:"));
+        return new ResourceURLContent(new URL(contentFile), contentFile.startsWith("jar:"));
       } catch (MalformedURLException ex1) {
         if (this.contentContext != null) {
           try {
-            return this.contentContext.lookupContent(content);
+            return this.contentContext.lookupContent(contentFile);
           } catch (IOException ex2) {
-            throw new SAXException("Invalid content " + content, ex2);
+            throw new SAXException("Invalid content " + contentFile, ex2);
+          }
+        } else if (catalogId != null && this.preferences != null) {
+          // Try to find a resource matching contentFile among catalogs
+          for (FurnitureCategory category : this.preferences.getFurnitureCatalog().getCategories()) {
+            for (CatalogPieceOfFurniture piece : category.getFurniture()) {
+              if (catalogId.equals(piece.getId())) {
+                if (isSameContent(contentFile, piece.getIcon())) {
+                  return piece.getIcon();
+                } else if (isSameContent(contentFile, piece.getPlanIcon())) {
+                  return piece.getPlanIcon();
+                } else if (isSameContent(contentFile, piece.getModel())) {
+                  return piece.getModel();
+                }
+              }
+            }
+          }
+          for (TexturesCategory category : this.preferences.getTexturesCatalog().getCategories()) {
+            for (CatalogTexture texture : category.getTextures()) {
+              if (catalogId.equals(texture.getId()) 
+                  && isSameContent(contentFile, texture.getIcon())) {
+                return texture.getIcon();
+              } 
+            }
           }
-        } else {
-          throw new SAXException("Missing URL base", ex1);
         }
+        throw new SAXException("Missing URL base", ex1);
       }
     } else {
       return null;
     }
   }
+  
+  /**
+   * Returns <code>true</code> if a content matches a given string.
+   */
+  private boolean isSameContent(String contentFile, Content content) {
+    if (content instanceof ResourceURLContent) {
+      ResourceURLContent resourceContent = (ResourceURLContent)content;
+      return resourceContent.isJAREntry() && resourceContent.getJAREntryName().equals(contentFile)
+          || !resourceContent.isJAREntry() && resourceContent.getURL().toString().endsWith("/" + contentFile);
+    } else {
+      return false;
+    }
+  }
 
   /**
    * Sets the home that will be updated by this handler.
